#include <gecode/driver.hh>
#include <gecode/int.hh>
#include <gecode/float.hh>
#include <gecode/set.hh>
#include <gecode/minimodel.hh>
#include <stdio.h>
#include <iostream>
#include <fstream>
#include <time.h>
#include <sys/resource.h>
#include <vector>
#include <math.h>
#include <algorithm> 


using namespace Gecode;
using namespace std;


class IGConstraint : public Propagator {
	protected:
		ViewArray<Int::IntView> x;
		ViewArray<Int::IntView> y;


	public:
		// posting
		IGConstraint(Home home, ViewArray<Int::IntView>& x0, ViewArray<Int::IntView>& y0) 	// maybe we use ADVISOR to reduce number of propagation
			: Propagator(home), x(x0), y(y0)  {
			x.subscribe(home,*this,Int::PC_INT_DOM);
		}
		static ExecStatus post(Home home, 
			                     ViewArray<Int::IntView>& x0, ViewArray<Int::IntView>& y0) {
			(void) new (home) IGConstraint(home,x0,y0);
			return ES_OK;
		}
		// disposal
		virtual size_t dispose(Home home) {
			x.cancel(home,*this,Int::PC_INT_DOM);

			(void) Propagator::dispose(home);
			return sizeof(*this);
		}
		// copying
		IGConstraint(Space& home, bool share, IGConstraint& p) 
			: Propagator(home,share,p) {
				x.update(home, share, p.x);
				y.update(home, share, p.y);
		}
		virtual Propagator* copy(Space& home, bool share) {
			return new (home) IGConstraint(home,share,*this);
		}
		// cost computation
		virtual PropCost cost(const Space&, const ModEventDelta&) const {
			return PropCost::binary(PropCost::HI);
		}
		// propagation
		virtual ExecStatus propagate(Space& home, const ModEventDelta&)  {


			int k = x.size();
			int n = y.size();

			int check[n];
			for (int i = 0; i < n; i++)
				check[i] = 0;


			for (int i = 0; i < k; i++)
				if (!x[i].assigned())			
					return ES_OK;
				else					
					check[ x[i].val() ] = 1;

			for (int i = 0; i < n; i++)
				if (check[i] == 0)
					for (int j = 0; j < n; j++)
			  		  GECODE_ME_CHECK(y[j].nq(home, i));		


				  return ES_OK;
		}
};

	void IGConstraint(Home home, IntVarArgs x, IntVarArgs y) {
		// constraint post function
    ViewArray<Int::IntView> vx(home, x);
    ViewArray<Int::IntView> vy(home, y);

		if (IGConstraint::post(home, vx, vy) != ES_OK)
			home.fail();
	}

